#!/usr/bin/env python
# -*- coding: utf-8 -*- #

# original source: https://github.com/lowtechmag/solar-plugins/tree/master/dither
# hitherdither: https://github.com/hbldh/hitherdither

#adds advanced dithering effect to images
# Copyright (C) 2018  Roel Roscam Abbing

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os, imghdr
from PIL import Image
import hitherdither

DEFAULT_IMAGE_DIR = "images"
DEFAULT_DITHER_DIR = "dithers"
DEFAULT_THRESHOLD = [96, 96, 96] # this sets the contrast of the final image, rgb
DEFAULT_TRANSPARENCY= False
DEFAULT_TRANSPARENT_COLOR = [(125,125,125)]
DEFAULT_DITHER_PALETTE = [(25,25,25), (75,75,75),(125,125,125),(175,175,175),(225,225,225),(250,250,250)] # 6 tone palette\
DEFAULT_RESIZE_OUTPUT = True
DEFAULT_MAX_SIZE = (800,800)

#11 tone palette, heavier, more detail, less visible dither pattern
#[(0,0,0),(25,25,25),(50,50,50),(75,75,75),(100,100,100),(125,125,125),(150,150,150),(175,175,175),(200,200,200),(225,225,225),(250,250,250)]

#3 tone palette, lighter, heavier dithering effect
# [(0,0,0), (125,125,125), (250,250,250)]

in_path = "input"
out_path = "output"

transparency = False
if not os.path.exists(out_path):
    os.mkdir(out_path)

for dirpath, _, filenames in os.walk(in_path):
    for filename in filenames:
        file_, ext = os.path.splitext(filename)
        fn = os.path.join(dirpath,filename)
        of = os.path.join(out_path, filename.replace(ext,'.png'))
        if not os.path.exists(of) and imghdr.what(fn):
            print("Dither plugin: dithering {}".format(fn))

            img = Image.open(fn).convert('RGB')

            resize = True

            if resize:
                image_size = DEFAULT_MAX_SIZE
                img.thumbnail(image_size, Image.LANCZOS)

            palette = hitherdither.palette.Palette(DEFAULT_DITHER_PALETTE)

            threshold = DEFAULT_THRESHOLD

            #see hither dither documentation for different dithering algos
            img_dithered = hitherdither.ordered.bayer.bayer_dithering(img, palette, threshold, order=8)

            img_dithered.save(of, optimize=True)

